/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portletcontainer.pci.model;

import java.io.InputStream;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.exoplatform.commons.xml.ExoXPPParser;

import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.portletcontainer.pci.model.FilterMapping;

//NMTOKEN
//import org.xml.sax.Attributes;
//import org.xml.sax.helpers.AttributesImpl;


/**
 * Jul 8, 2004
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: XMLParser.java,v 1.1 2004/07/13 02:31:13 tuan08 Exp $
 */
public class XMLParser {

  static boolean newSpec = false;
  static PortletApp portletapp;
  
  static public PortletApp parse(InputStream is, boolean isnewer) throws Exception {
    newSpec = isnewer;
    ExoXPPParser xpp = ExoXPPParser.getInstance()  ;
    xpp.setInput(is, "UTF8");
    xpp.mandatoryNode("portlet-app") ;
    return readPortletApp(xpp) ;
  }  
  
  static public PortletApp readPortletApp(ExoXPPParser xpp ) throws Exception {
    portletapp = new PortletApp();
    portletapp.setVer2(newSpec);
    portletapp.setVersion(xpp.getNodeAttributeValue("version")) ;
    portletapp.setId(xpp.getNodeAttributeValue("id")) ;
    while(xpp.node("portlet")) portletapp.addPortlet(readPortlet(xpp)) ;
    while(xpp.node("custom-portlet-mode")) portletapp.addCustomPortletMode(readCustomPortletMode(xpp)) ;
    while(xpp.node("custom-window-state")) portletapp.addCustomWindowState(readCustomWindowState(xpp)) ;
    while(xpp.node("user-attribute")) portletapp.addUserAttribute(readUserAttribute(xpp)) ;
    while(xpp.node("security-constraint")) portletapp.addSecurityConstraint(readSecurityConstraint(xpp)) ;
    // portlet api 2.0
    if (newSpec) {
      if(xpp.node("resource-bundle")) portletapp.setResourceBundle(xpp.getContent()) ;
      while(xpp.node("filter")) portletapp.addFilter(readFilter(xpp)) ;
      while(xpp.node("filter-mapping")) portletapp.addFilterMapping(readFilterMapping(xpp)) ;
      fillPredefindeEvents(portletapp);
      while(xpp.node("event-definition")) portletapp.addEventDefinition(readEventDefinition(xpp)) ;
      while(xpp.node("public-render-parameter")) portletapp.addPublicRenderParameter(readPublicRenderParameter(xpp)) ;
      while(xpp.node("container-runtime-option")) portletapp.addContainerRuntimeOption(readContainerRuntimeOption(xpp)) ;
    }
    return portletapp ;
  }

  static public Portlet readPortlet(ExoXPPParser xpp ) throws Exception {
    Portlet p = new Portlet()  ;
    p.setId(xpp.getNodeAttributeValue("id")) ;
    while(xpp.node("description")) p.addDescription(readDescription(xpp)) ;
    xpp.mandatoryNode("portlet-name"); p.setPortletName(xpp.getContent()) ;
    while(xpp.node("display-name")) p.addDisplayName(readDisplayName(xpp)) ;
    xpp.mandatoryNode("portlet-class"); p.setPortletClass(xpp.getContent()) ;
    while(xpp.node("init-param")) p.addInitParam(readInitParam(xpp)) ;
    if (newSpec) {
      if(xpp.node("caching")) {
        p.setCaching(xpp.node("expiration-cache")?
          Integer.parseInt(xpp.getContent()):
          null);
        p.setGlobalCache(xpp.node("scope")?
          (xpp.getContent().equalsIgnoreCase("public")? // in the 1-st spec was: global 
            "true":
            "false"):
          "false");
      }
    } else {
      if(xpp.node("expiration-cache")) p.setCaching(Integer.parseInt(xpp.getContent())) ;
    }
    xpp.mandatoryNode("supports"); p.setSupports(readSupports(xpp)) ;
    while(xpp.node("supports")) p.addSupports(readSupports(xpp)) ;
    while(xpp.node("supported-locale")) p.addSupportedLocale(xpp.getContent()) ;
    boolean chosen = false;
    if(xpp.node("resource-bundle")) {
      p.setResourceBundle(xpp.getContent()) ;
      chosen = true;
    }
    if(xpp.node("portlet-info")) {
      p.setPortletInfo(readPortletInfo(xpp)) ;
      chosen = true;
    }
    if ( !chosen ) {
      throw new Exception("expect tag name 'resource-bundle' or 'portlet-info'");
    }
    if(xpp.node("portlet-preferences")) p.setPortletPreferences(readPortletPreferences(xpp)) ;
    while(xpp.node("security-role-ref")) p.addSecurityRoleRef(readSecurityRoleRef(xpp)) ;
    // exo tag only for portlet 1.0
    if (!newSpec) {
      if(xpp.node("global-cache")) p.setGlobalCache(xpp.getContent()) ;
    }
    // portlet api 2.0
    if (newSpec) {
      while(xpp.node("supported-processing-event")) p.addSupportedProcessingEvent(new QName(xpp.getContent()));
      while(xpp.node("supported-publishing-event")) p.addSupportedPublishingEvent(new QName(xpp.getContent()));
      while(xpp.node("supported-public-render-parameter")) p.addSupportedPublicRenderParameter(xpp.getContent());
      while(xpp.node("container-runtime-option")) p.addContainerRuntimeOption(readContainerRuntimeOption(xpp)); 
    }
    return p ;
  }

  static public CustomPortletMode readCustomPortletMode(ExoXPPParser xpp ) throws Exception {
    CustomPortletMode mode = new CustomPortletMode()  ;
    mode.setId(xpp.getNodeAttributeValue("id")) ;
     while(xpp.node("description")) mode.addDescription(readDescription(xpp)) ;
     xpp.mandatoryNode("portlet-mode"); mode.setPortletMode(xpp.getContent()) ;
     if (xpp.node("portal-managed")) {
       mode.setPortalManaged(xpp.getContent()) ;
     } else {
       mode.setPortalManaged(Boolean.TRUE.toString());
     }
     if (mode.getPortalManaged() == Boolean.FALSE.toString()) {
       xpp.mandatoryNode("decoration-name"); mode.setResourceID(xpp.getContent()) ;
     } else {
       if (xpp.node("decoration-name")) mode.setResourceID(xpp.getContent()) ;       
     }
    return mode ;
  }

  static public CustomWindowState readCustomWindowState(ExoXPPParser xpp ) throws Exception {
    CustomWindowState state = new CustomWindowState()  ;
    state.setId(xpp.getNodeAttributeValue("id")) ;
    while(xpp.node("description")) state.addDescription(readDescription(xpp)) ;
    xpp.mandatoryNode("window-state"); state.setWindowState(xpp.getContent()) ;
    if (!newSpec){
      List portlets = portletapp.getPortlet();
      for (int i = 0; i < portlets.toArray().length; i++) {
        Portlet portlet = (Portlet)portlets.toArray()[i];
        List supports = portlet.getSupports();
        for (int j = 0; j < supports.toArray().length; j++) {
          Supports support = (Supports)supports.toArray()[j];
          support.addWindowState(state.getWindowState());
        }
      }      
    }
    return state ;
  }

  static public UserAttribute readUserAttribute(ExoXPPParser xpp ) throws Exception {
    UserAttribute att = new UserAttribute()  ;
    att.setId(xpp.getNodeAttributeValue("id")) ;
    while(xpp.node("description")) att.addDescription(readDescription(xpp)) ;
    xpp.mandatoryNode("name"); att.setName(xpp.getContent()) ;
    return att ;
  }

  static public SecurityConstraint readSecurityConstraint(ExoXPPParser xpp ) throws Exception {
    SecurityConstraint sc = new SecurityConstraint()  ;
    sc.setId(xpp.getNodeAttributeValue("id")) ;
    if(xpp.node("displayName"))sc.addDisplayName(readDisplayName(xpp)) ;
    // portlet api 2.0
    if(xpp.node("display-name"))sc.addDisplayName(readDisplayName(xpp)) ;
    //
    xpp.mandatoryNode("portlet-collection"); sc.setPortletCollection(readPortletCollection(xpp)) ;
    xpp.mandatoryNode("user-data-constraint"); sc.setUserDataConstraint(readUserDataConstraint(xpp)) ;
    return sc ;
  }

  static public Description readDescription(ExoXPPParser xpp) throws Exception {
    Description desc = new Description() ;
    desc.setLang(xpp.getNodeAttributeValue("xml:lang")) ;
    desc.setDescription(xpp.getContent()) ;
    return desc ;
  }

  static public DisplayName readDisplayName(ExoXPPParser xpp) throws Exception {
    DisplayName name = new DisplayName() ;
    name.setLang(xpp.getNodeAttributeValue("xml:lang")) ;
    name.setDisplayName(xpp.getContent()) ;
    return name ;
  }

  static public InitParam readInitParam(ExoXPPParser xpp) throws Exception {
    InitParam param = new InitParam() ;
    while(xpp.node("description")) param.addDescription(readDescription(xpp)) ;
    xpp.mandatoryNode("name"); param.setName(xpp.getContent()) ;
    xpp.mandatoryNode("value"); param.setValue(xpp.getContent()) ;
    return param ;
  }

  static public Supports readSupports(ExoXPPParser xpp) throws Exception {
    Supports supports = new Supports() ;
    supports.setId(xpp.getNodeAttributeValue("id")) ;
    xpp.mandatoryNode("mime-type"); supports.setMimeType(xpp.getContent()) ;
    // getting portlet modes
    List<String> portletMode = new ArrayList<String>();
    while(xpp.node("portlet-mode")) portletMode.add(xpp.getContent());
    if (!portletMode.contains(PortletMode.VIEW.toString()))
      supports.addPortletMode(PortletMode.VIEW.toString());
    Iterator iterModes = portletMode.iterator();
    while (iterModes.hasNext()) {
      supports.addPortletMode(((String) iterModes.next()) );
    }    
    if (newSpec) {
      // getting window states
      List<String> windowState = new ArrayList<String>();
      while(xpp.node("window-state")) windowState.add(xpp.getContent());
      if (!windowState.contains(WindowState.MINIMIZED.toString())) 
        supports.addWindowState(WindowState.MINIMIZED.toString());
      if (!windowState.contains(WindowState.NORMAL.toString())) 
        supports.addWindowState(WindowState.NORMAL.toString());
      if (!windowState.contains(WindowState.MAXIMIZED.toString())) 
        supports.addWindowState(WindowState.MAXIMIZED.toString());
      Iterator iterStates = windowState.iterator();
      while (iterStates.hasNext()) {
        supports.addWindowState(((String) iterStates.next()) );
      }
    } else {
      supports.addWindowState(WindowState.MINIMIZED.toString());
      supports.addWindowState(WindowState.NORMAL.toString());
      supports.addWindowState(WindowState.MAXIMIZED.toString());
    }
    return supports ;
  }

  static public PortletInfo readPortletInfo(ExoXPPParser xpp) throws Exception {
    PortletInfo info = new PortletInfo() ;
    if(xpp.node("title")) info.setTitle(xpp.getContent()) ;
    if(xpp.node("short-title")) info.setShortTitle(xpp.getContent()) ;
    if(xpp.node("keywords")) info.setKeywords(xpp.getContent()) ;
    return info ;
  }

  static public ExoPortletPreferences readPortletPreferences(ExoXPPParser xpp) throws Exception {
    ExoPortletPreferences prefs = new ExoPortletPreferences() ;
    prefs.setId(xpp.getNodeAttributeValue("id")) ;
    while(xpp.node("preference"))prefs.addPreference(readPreference(xpp)) ;
    if(xpp.node("preferences-validator")) prefs.setPreferencesValidator(xpp.getContent()) ;
    return prefs ;
  }

  static public Preference readPreference(ExoXPPParser xpp) throws Exception {
    Preference pref = new Preference() ;
    pref.setId(xpp.getNodeAttributeValue("id")) ;
    xpp.mandatoryNode("name"); pref.setName(xpp.getContent()) ;
    while(xpp.node("value"))pref.addValue(xpp.getContent()) ;
    if(xpp.node("read-only")) pref.setReadOnly(xpp.getContent()) ;
    return pref ;
  }

  static public SecurityRoleRef readSecurityRoleRef(ExoXPPParser xpp) throws Exception {
    SecurityRoleRef ref = new SecurityRoleRef() ;
    // portlet api 2.0
    ref.setId(xpp.getNodeAttributeValue("id")) ;
    while(xpp.node("description")) ref.addDescription(readDescription(xpp)) ;
    xpp.mandatoryNode("role-name"); ref.setRoleName(xpp.getContent()) ;
    /*
    String roleName = ref.getRoleName();
    AttributesImpl rn = new AttributesImpl();
    org.xml.sax.Attributes at = new AttributesImpl();
    System.out.println(">>> EXOMAN XMLParser.readSecurityRoleRef BEFORE 11111111111111 = ");
    rn.addAttribute(null,"role-name","role-name","NMTOKEN","!@#$$@#%$&");
    System.out.println(">>> EXOMAN XMLParser.readSecurityRoleRef AFTER 22222222222222 = ");   
    */
    if(xpp.node("role-link")) ref.setRoleLink(xpp.getContent()) ;
    return ref ;
  }

  static public Filter readFilter(ExoXPPParser xpp) throws Exception {
    Filter filter = new Filter() ;
    xpp.mandatoryNode("filter-name"); filter.setFilterName(xpp.getContent()) ;
    xpp.mandatoryNode("filter-class"); filter.setFilterClass(xpp.getContent()) ;
    filter.addLifecycle(Filter.ALL);
    xpp.mandatoryNode("lifecycle"); filter.addLifecycle(parseLifecycle(xpp.getContent(), filter.getFilterName()));
    while (xpp.node("lifecycle")) filter.addLifecycle(parseLifecycle(xpp.getContent(), filter.getFilterName()));
    while (xpp.node("init-param")) filter.addInitParam(readInitParam(xpp));
    return filter ;
  }
  
  static public Integer parseLifecycle(String cycle, String filterName) {
    if (cycle.equalsIgnoreCase("ACTION_PHASE"))
      return(Filter.ACTION);
    else
      if (cycle.equalsIgnoreCase("EVENT_PHASE"))
        return(Filter.EVENT);
    else
    if (cycle.equalsIgnoreCase("RENDER_PHASE"))
      return(Filter.RENDER);
    else
    if (cycle.equalsIgnoreCase("RESOURCE_PHASE"))
      return(Filter.RESOURCE);
    else {
      Log log = ExoLogger.getLogger("org.exoplatform.services.portletcontainer");
      log.error("WHILE PARSING portlet.xml the filter '" + filterName + "' doesn't have proper lifecycle.");
    }
    return null;
  }

  static public FilterMapping readFilterMapping(ExoXPPParser xpp) throws Exception {
    FilterMapping filterMapping = new FilterMapping() ;
    xpp.mandatoryNode("filter-name"); filterMapping.setFilterName(xpp.getContent()) ;
    xpp.mandatoryNode("portlet-name"); filterMapping.addPortletName(xpp.getContent()) ;
    while(xpp.node("portlet-name")) filterMapping.addPortletName(xpp.getContent()) ;
    return filterMapping;
  }

  static public PortletCollection readPortletCollection(ExoXPPParser xpp) throws Exception {
    PortletCollection collection = new PortletCollection() ;
    xpp.mandatoryNode("portlet-name"); collection.addPortletName(xpp.getContent()) ;
    while(xpp.node("portlet-name")) collection.addPortletName(xpp.getContent()) ;
    return collection ;
  }

  static public UserDataConstraint readUserDataConstraint(ExoXPPParser xpp) throws Exception {
    UserDataConstraint u = new UserDataConstraint() ;
    u.setId(xpp.getNodeAttributeValue("id")) ;
    while(xpp.node("description")) u.addDescription(readDescription(xpp)) ;
    xpp.mandatoryNode("transport-guarantee"); u.setTransportGuarantie(xpp.getContent()) ;
    return u ;
  }

  // portlet api 2.0

  static public EventDefinition readEventDefinition(ExoXPPParser xpp) throws Exception {
    EventDefinition ed = new EventDefinition();
    ed.setId(xpp.getNodeAttributeValue("id"));
    while (xpp.node("description")) ed.addDescription(readDescription(xpp)) ;
    xpp.mandatoryNode("name"); ed.setPrefferedName(new QName(xpp.getContent()));
    while (xpp.node("alias")) ed.addAlias(new QName(xpp.getContent()));
    if (xpp.node("java-class")) ed.setJavaClass(xpp.getContent());
    return ed;
  }

  static public void fillPredefindeEvents(PortletApp app) throws Exception {
    EventDefinition ed = new EventDefinition();
    ed.setPrefferedName(new QName("urn:oasis:names:tc:wsrp:v2:types", "wsrp:eventHandlingFailed"));
    app.addEventDefinition(ed);
    ed = new EventDefinition();
    ed.setPrefferedName(new QName("urn:oasis:names:tc:wsrp:v2:types", "wsrp:newNavigationalContextScope"));
    app.addEventDefinition(ed);
  }

  static public PublicRenderParameter readPublicRenderParameter(ExoXPPParser xpp) throws Exception {
    PublicRenderParameter srp = new PublicRenderParameter();
    srp.setId(xpp.getNodeAttributeValue("id"));
    while (xpp.node("description")) srp.addDescription(readDescription(xpp));
    xpp.mandatoryNode("identifier"); srp.setPrefferedName(xpp.getContent());
    xpp.mandatoryNode("name"); srp.addName(new QName(xpp.getContent()));
    while (xpp.node("name")) srp.addName(new QName(xpp.getContent()));
    return srp;
  }
  
  static public Map readContainerRuntimeOption(ExoXPPParser xpp) throws Exception {
    HashMap<String,String[]> options = new HashMap<String,String[]>();
    xpp.mandatoryNode("name"); String name = xpp.getContent();
    ArrayList<String> values = new ArrayList<String>();
    if (xpp.node("value"))
      values.add(xpp.getContent());
    else {
      options.put(name,null);
      return options;
    }
    while (xpp.node("value")) 
      values.add(xpp.getContent());
    options.put(name, (String[])values.toArray());
    return options;
  }

}
