/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.       *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portletcontainer.plugins.pc.aop;

import java.util.Iterator;

import javax.portlet.PortletContext;

import org.exoplatform.services.portletcontainer.plugins.pc.Constants;
import org.exoplatform.services.portletcontainer.plugins.pc.filter.PortletFilterChainImpl;
import org.exoplatform.services.portletcontainer.plugins.pc.filter.PortletFilterConfigImpl;
import org.exoplatform.services.portletcontainer.plugins.pc.filter.PortletFilterWrapper;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.PortletRequestImp;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.PortletResponseImp;

/*
 * @author: Benjamin Mestrallet
 * @author: Tuan Nguyen
 * @author: Roman Pedchenko
 */

public class PortletFilterCommand extends BaseCommandUnit {

  protected Object render(RenderExecutionContext rcontext)  throws Throwable {
    log_.debug("--> render method, call portlet filter aspect");
    
    PortletRequestImp req = (PortletRequestImp) rcontext.request_;
    PortletResponseImp res = (PortletResponseImp) rcontext.response_;
    PortletContext portletContext = req.getPortletConfig().getPortletContext();
    String portletAppName = req.getPortletWindowInternal().getWindowID().getPortletApplicationName();
    PortletFilterChainImpl chain = (PortletFilterChainImpl) req.getPortletDatas().getFilterChain();
    for (Iterator iterator = chain.getFiltersIterator(); iterator.hasNext();) {
      PortletFilterWrapper portletFilter = (PortletFilterWrapper) iterator.next();
      portletFilter.init(new PortletFilterConfigImpl(portletFilter.getFilterName(), portletFilter.getInitParam(), portletContext));
    }
    chain.invoke(req, res, Constants.renderInt);
    return rcontext.executeNextUnit();
  }

  protected Object processAction(ActionExecutionContext acontext)  throws Throwable {
    log_.debug("--> processAction method, call portlet filter aspect");
    
    PortletRequestImp req = (PortletRequestImp) acontext.request_;
    PortletResponseImp res = (PortletResponseImp) acontext.response_;
    PortletContext portletContext = req.getPortletConfig().getPortletContext();
    String portletAppName = req.getPortletWindowInternal().getWindowID().getPortletApplicationName();
    PortletFilterChainImpl chain = (PortletFilterChainImpl) req.getPortletDatas().getFilterChain();
    for (Iterator iterator = chain.getFiltersIterator(); iterator.hasNext();) {
      PortletFilterWrapper portletFilter = (PortletFilterWrapper) iterator.next();
      portletFilter.init(new PortletFilterConfigImpl(portletFilter.getFilterName(), portletFilter.getInitParam(), portletContext));
    }
    chain.invoke(req, res, Constants.actionInt);
    return acontext.executeNextUnit();
  }

  protected Object serveResource(ResourceExecutionContext rcontext)  throws Throwable {
    log_.debug("--> serveResource method, call portlet filter aspect");
    PortletRequestImp req = (PortletRequestImp) rcontext.request_;
    PortletResponseImp res = (PortletResponseImp) rcontext.response_;
    PortletContext portletContext = req.getPortletConfig().getPortletContext();
    String portletAppName = req.getPortletWindowInternal().getWindowID().getPortletApplicationName();
    PortletFilterChainImpl chain = (PortletFilterChainImpl) req.getPortletDatas().getFilterChain();
    for (Iterator iterator = chain.getFiltersIterator(); iterator.hasNext();) {
      PortletFilterWrapper portletFilter = (PortletFilterWrapper) iterator.next();
      portletFilter.init(new PortletFilterConfigImpl(portletFilter.getFilterName(), portletFilter.getInitParam(), portletContext));
    }
    chain.invoke(req, res, Constants.resourceInt);
    return rcontext.executeNextUnit();
  }

  protected Object processEvent(EventExecutionContext econtext)  throws Throwable {
    log_.debug("--> processEvent method, call portlet filter aspect");
    
    PortletRequestImp req = (PortletRequestImp) econtext.request_;
    PortletResponseImp res = (PortletResponseImp) econtext.response_;
    PortletContext portletContext = req.getPortletConfig().getPortletContext();
    String portletAppName = req.getPortletWindowInternal().getWindowID().getPortletApplicationName();
    PortletFilterChainImpl chain = (PortletFilterChainImpl) req.getPortletDatas().getFilterChain();
    for (Iterator iterator = chain.getFiltersIterator(); iterator.hasNext();) {
      PortletFilterWrapper portletFilter = (PortletFilterWrapper) iterator.next();
      portletFilter.init(new PortletFilterConfigImpl(portletFilter.getFilterName(), portletFilter.getInitParam(), portletContext));
    }
    chain.invoke(req, res, Constants.eventInt);
    return econtext.executeNextUnit();
  }

}