/**
 * Copyright 2001-2003 The eXo platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

package org.exoplatform.services.portletcontainer.plugins.pc.filter;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import javax.portlet.PortletException;
import javax.portlet.ActionRequest;
import javax.portlet.EventRequest;
import javax.portlet.RenderRequest;
import javax.portlet.ResourceRequest;
import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.filter.FilterChain;
import javax.portlet.filter.FilterConfig;

//import org.exoplatform.services.portletcontainer.pci.model.FilterMapping;
import org.exoplatform.services.portletcontainer.pci.model.Filter;
//import java.util.Collection;
//import java.util.Iterator;
//import java.io.IOException;

/**
 * Created by The eXo Platform SARL
 * Author : Roman Pedchenko
 *          roman.pedchenko@exoplatform.com.ua
 */

public class PortletFilterWrapper implements javax.portlet.filter.Filter {

  private String filterName;
  private String filterClass;
  private List initParams;
  private javax.portlet.filter.Filter filter;
  private List lifecycle;
  
  public PortletFilterWrapper(String filterName, String filterClass, List initParams, List lifecycle) {
    this.filterName = filterName;
    this.filterClass = filterClass;
    this.initParams = initParams;
    this.lifecycle = lifecycle;
    filter = null;
  }

  public String getFilterName() {
    return filterName;
  }

  public List getInitParam() {
    return initParams;
  }

  public void init(FilterConfig config) throws PortletException, ClassNotFoundException, 
      IllegalAccessException, InstantiationException {
    if (filter == null) {
      filter = (javax.portlet.filter.Filter) Thread.currentThread().getContextClassLoader().loadClass(filterClass).newInstance();
      filter.init(config);
    }
  }

  public void doFilter(PortletRequest request, PortletResponse response, FilterChain chain) throws IOException, PortletException {
    if (isLifecycleSupported(request)) {
      filter.doFilter(request, response, chain);
    }
  }

  private boolean isLifecycleSupported(PortletRequest request) {
    Iterator cycles = lifecycle.iterator();
    while (cycles.hasNext()) {
      int cycle = ((Integer) cycles.next()).intValue();
      if ((cycle == Filter.ALL) ||
        ((request instanceof ActionRequest) && (cycle == Filter.ACTION)) ||
        ((request instanceof EventRequest) && (cycle == Filter.EVENT)) ||
        ((request instanceof RenderRequest) && (cycle == Filter.RENDER)) ||
        ((request instanceof ResourceRequest) && (cycle == Filter.RESOURCE))
      ) return true;
    }
    return false;    
  }
  
  public void destroy() {
    filter.destroy();
  }

}
