/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

/**
 * Created by The eXo Platform SARL
 * Author : Roman Pedchenko <roman.pedchenko@exoplatform.com.ua>
 */

package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp;


import java.io.*;
import java.util.*;
import javax.portlet.ClientDataRequest;
//import javax.servlet.http.HttpServletRequest;
//import org.exoplatform.Constants;
//import org.exoplatform.container.ExoContainer;

public abstract class ClientDataRequestImp extends PortletRequestImp implements ClientDataRequest {

  private boolean isCallGetReader;
  private boolean isCallGetStream;
  private boolean areParamsTouched;

  public ClientDataRequestImp(RequestContext reqCtx) {
    super(reqCtx);
    isCallGetReader = false;
    isCallGetStream = false;
    areParamsTouched = false;
//    enc = super.getCharacterEncoding();
  }

  public String getParameter(String param) {
    areParamsTouched = true;
    if (this.isCallGetReader || this.isCallGetStream)
      return null;
    return super.getParameter(param);
  }

  public Enumeration getParameterNames() {
    areParamsTouched = true;
    if (this.isCallGetReader || this.isCallGetStream)
      return new StringTokenizer("");
    return super.getParameterNames();
  }

  public String[] getParameterValues(String s) {
    areParamsTouched = true;
    if (this.isCallGetReader || this.isCallGetStream)
      return null;
    return super.getParameterValues(s);
  }

  public Map getParameterMap() {
    areParamsTouched = true;
    if (this.isCallGetReader || this.isCallGetStream)
      return new HashMap();
    return super.getParameterMap();
  }

  public byte[] makeFormDataByteArray() {
    if (areParamsTouched)
      return new byte[0];
    try {
      StringBuffer r = new StringBuffer();
      boolean touched = false;
      for (Enumeration e = super.getParameterNames(); e.hasMoreElements(); ) {
        String n = (String) e.nextElement();
        String[] v = super.getParameterValues(n);
        if (v != null && v.length > 0) {
          for (int i = 0; i < v.length; i++) {
            if (touched)
              r.append("&");
            else
              touched = true;
            r.append(n + "=" + java.net.URLEncoder.encode(v[i], (this.enc != null)?this.enc:"utf-8"));
          }
        }
      }
      return r.toString().getBytes("us-ascii");
    } catch (UnsupportedEncodingException e) {
      return new byte[0];
    }
  }

  public java.io.InputStream getPortletInputStream() throws java.io.IOException {
    if (this.isCallGetReader) {
      throw new IllegalStateException("getPortletInputStream() cannot be called, when getReader() method has been called");
    }
    if (this.isCallGetStream) {
      throw new IllegalStateException("getPortletInputStream() cannot be called twice");
    }
    String contentType = getHeader("content-type");
    if ("application/x-www-form-urlencoded".equals(contentType)) {
      if (!reqCtx.getPortletDatas().getApplication().getVer2() || !getMethod().equalsIgnoreCase("post"))
        throw new IllegalStateException("content type cannot be application/x-www-form-urlencoded");
      this.isCallGetStream = true;
      return new ByteArrayInputStream(makeFormDataByteArray());
    }
    this.isCallGetStream = true;
    return super.getInputStream();
  }

  public java.io.BufferedReader getReader()
      throws java.io.UnsupportedEncodingException, java.io.IOException {
    if (this.isCallGetReader) {
      throw new IllegalStateException("getPortletInputStream() cannot be called twice");
    }
    if (this.isCallGetStream) {
      throw new IllegalStateException("getReader() cannot be called, when getPortletInputStream() method has been called");
    }
    String contentType = getHeader("content-type");
    if ("application/x-www-form-urlencoded".equals(contentType)) {
      if (!reqCtx.getPortletDatas().getApplication().getVer2() || !getMethod().equalsIgnoreCase("post"))
        throw new IllegalStateException("content type cannot be application/x-www-form-urlencoded");
      this.isCallGetReader = true;
      return new BufferedReader(new CharArrayReader((new String(makeFormDataByteArray(), "us-ascii")).toCharArray()));
    }
    this.isCallGetReader = true;
    return super.getReader();
  }

  public void setCharacterEncoding(String enc) throws java.io.UnsupportedEncodingException {
    if (this.isCallGetReader || this.isCallGetStream) {
      throw new IllegalStateException("This method cannot be called, when getReader() method has been called");
    }
    super.setCharacterEncoding(enc);
    this.enc = enc;
    this.encodingModified = true;
  }

  public String getCharacterEncoding() {
    return super.getCharacterEncoding();
  }
  
  public String getContentType() {
    return super.getContentType();
  }
  
  public int getContentLength() {
    return super.getContentLength();
  }
  
  public String getMethod() {
    return super.getMethod();
  }
  
  public abstract String getLifecyclePhase();
  
}
