/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/
package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.EventRequest;
import javax.portlet.EventResponse;
//import javax.portlet.FragmentRequest;
//import javax.portlet.FragmentResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.portlet.ResourceRequest;
import javax.portlet.ResourceResponse;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponseWrapper;

import org.apache.commons.logging.Log;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.portletcontainer.plugins.pc.PortletContainerConf;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.helpers.CustomRequestWrapper;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.helpers.CustomResponseWrapper;
import org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp.helpers.NestedResponseWrapper;

/**
 * Created by The eXo Platform SARL
 * Author : Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 * Date: Jul 25, 2003
 * Time: 6:46:52 PM
 */
public class PortletRequestDispatcherImp implements PortletRequestDispatcher {

  private String path;
  private RequestDispatcher dispatcher;
  private Log log;
  protected ExoContainer cont;

  public PortletRequestDispatcherImp(ExoContainer cont, RequestDispatcher dispatcher, String path) {
    this.dispatcher = dispatcher;
    this.path = path;
    this.cont = cont;
    this.log = ((LogService) cont.getComponentInstanceOfType(LogService.class)).
        getLog("org.exoplatform.services.portletcontainer");
  }

  public void include(RenderRequest renderRequest, RenderResponse renderResponse)
      throws PortletException, IOException {
    CustomRequestWrapper requestWrapper = null;    
    PortletContainerConf conf = (PortletContainerConf) cont.
        getComponentInstanceOfType(PortletContainerConf.class);
    try {      
      requestWrapper = 
        ((CustomRequestWrapper) ((HttpServletRequestWrapper)renderRequest).getRequest());
/*
      CustomResponseWrapper responseWrapper = 
        (CustomResponseWrapper) ((HttpServletResponseWrapper)response).getResponse();
*/
      CustomResponseWrapper realResponseWrapper = 
        (CustomResponseWrapper) ((HttpServletResponseWrapper)renderResponse).getResponse();
      NestedResponseWrapper responseWrapper = new NestedResponseWrapper(realResponseWrapper);
            
      renderRequest.setAttribute("javax.portlet.config",((RenderRequestImp)renderRequest).getPortletConfig());
      renderRequest.setAttribute("javax.portlet.request", renderRequest);
      renderRequest.setAttribute("javax.portlet.response", renderResponse);
      renderRequest.setAttribute(PortletRequest.LIFECYCLE_PHASE, PortletRequest.RENDER_PHASE);
      responseWrapper.flushBuffer();
      requestWrapper.setRedirected(true);
      requestWrapper.setRedirectedPath(path);
      requestWrapper.setContextPath(renderRequest.getContextPath());
      if (dispatcher != null)
        dispatcher.include(requestWrapper, responseWrapper);
      char[] ca = responseWrapper.getPortletContent();
      if (ca == null)
        ca = new char[0];
      if (realResponseWrapper.isStreamUsed()) {
        realResponseWrapper.getOutputStream().write((new String(ca)).getBytes("utf-8"));
      } else
        realResponseWrapper.getWriter().write(ca);
    } catch (ServletException e) {
      if (e.getRootCause() != null)
        log.error("Root cause of the exception", e.getRootCause());
      log.error("Problems occur when using PortletDispatcher", e);
      throw new PortletException("Problems occur when using PortletDispatcher", e);
    } finally {
      renderRequest.removeAttribute("javax.portlet.config");
      renderRequest.removeAttribute("javax.portlet.request");
      renderRequest.removeAttribute("javax.portlet.response");   
      renderRequest.removeAttribute(PortletRequest.LIFECYCLE_PHASE); 
      if(requestWrapper != null)
        requestWrapper.setRedirected(false);
    }    
  }
  
  public void include(ActionRequest actionRequest, ActionResponse actionResponse) throws PortletException, IOException {
    CustomRequestWrapper requestWrapper = null;    
    PortletContainerConf conf = (PortletContainerConf) cont.
        getComponentInstanceOfType(PortletContainerConf.class);
    try {      
      requestWrapper = 
        ((CustomRequestWrapper) ((HttpServletRequestWrapper)actionRequest).getRequest());
/*
      CustomResponseWrapper responseWrapper = 
        (CustomResponseWrapper) ((HttpServletResponseWrapper)response).getResponse();
*/
      CustomResponseWrapper realResponseWrapper = 
        (CustomResponseWrapper) ((HttpServletResponseWrapper)actionResponse).getResponse();
      NestedResponseWrapper responseWrapper = new NestedResponseWrapper(realResponseWrapper);
            
      actionRequest.setAttribute("javax.portlet.config",((ActionRequestImp)actionRequest).getPortletConfig());
      actionRequest.setAttribute("javax.portlet.request", actionRequest);
      actionRequest.setAttribute("javax.portlet.response", actionResponse);
      actionRequest.setAttribute(PortletRequest.LIFECYCLE_PHASE, PortletRequest.ACTION_PHASE);
      responseWrapper.flushBuffer();
      requestWrapper.setRedirected(true);
      requestWrapper.setRedirectedPath(path);
      requestWrapper.setContextPath(actionRequest.getContextPath());
      if (dispatcher != null)
        dispatcher.include(requestWrapper, responseWrapper);
      char[] ca = responseWrapper.getPortletContent();
      if (ca == null)
        ca = new char[0];
      if (realResponseWrapper.isStreamUsed()) {
        realResponseWrapper.getOutputStream().write((new String(ca)).getBytes());
      } else
        realResponseWrapper.getWriter().write(ca);
    } catch (ServletException e) {
      if (e.getRootCause() != null)
        log.error("Root cause of the exception", e.getRootCause());
      log.error("Problems occur when using PortletDispatcher", e);
      throw new PortletException("Problems occur when using PortletDispatcher", e);
    } finally {
      actionRequest.removeAttribute("javax.portlet.config");
      actionRequest.removeAttribute("javax.portlet.request");
      actionRequest.removeAttribute("javax.portlet.response");    
      actionRequest.removeAttribute(PortletRequest.LIFECYCLE_PHASE); 
      if(requestWrapper != null)
        requestWrapper.setRedirected(false);
    }    
  }
  
  public void include(ResourceRequest resourceRequest, ResourceResponse resourceResponse) throws PortletException, IOException {
    CustomRequestWrapper requestWrapper = null;    
    PortletContainerConf conf = (PortletContainerConf) cont.
        getComponentInstanceOfType(PortletContainerConf.class);
    try {      
      requestWrapper = 
        ((CustomRequestWrapper) ((HttpServletRequestWrapper)resourceRequest).getRequest());
/*
      CustomResponseWrapper responseWrapper = 
        (CustomResponseWrapper) ((HttpServletResponseWrapper)response).getResponse();
*/
      CustomResponseWrapper realResponseWrapper = 
        (CustomResponseWrapper) ((HttpServletResponseWrapper)resourceResponse).getResponse();
      NestedResponseWrapper responseWrapper = new NestedResponseWrapper(realResponseWrapper);
            
      resourceRequest.setAttribute("javax.portlet.config",((ResourceRequestImp)resourceRequest).getPortletConfig());
      resourceRequest.setAttribute("javax.portlet.request", resourceRequest);
      resourceRequest.setAttribute("javax.portlet.response", resourceResponse);
      resourceRequest.setAttribute(PortletRequest.LIFECYCLE_PHASE, PortletRequest.RESOURCE_SERVING_PHASE);
      responseWrapper.flushBuffer();
      requestWrapper.setRedirected(true);
      requestWrapper.setRedirectedPath(path);
      requestWrapper.setContextPath(resourceRequest.getContextPath());
      if (dispatcher != null)
        dispatcher.include(requestWrapper, responseWrapper);
      char[] ca = responseWrapper.getPortletContent();
      if (ca == null)
        ca = new char[0];
      if (realResponseWrapper.isStreamUsed()) {
        realResponseWrapper.getOutputStream().write((new String(ca)).getBytes());
      } else
        realResponseWrapper.getWriter().write(ca);
    } catch (ServletException e) {
      if (e.getRootCause() != null)
        log.error("Root cause of the exception", e.getRootCause());
      log.error("Problems occur when using PortletDispatcher", e);
      throw new PortletException("Problems occur when using PortletDispatcher", e);
    } finally {
      resourceRequest.removeAttribute("javax.portlet.config");
      resourceRequest.removeAttribute("javax.portlet.request");
      resourceRequest.removeAttribute("javax.portlet.response");     
      resourceRequest.removeAttribute(PortletRequest.LIFECYCLE_PHASE); 
      if(requestWrapper != null)
        requestWrapper.setRedirected(false);
    }    
  }
  
  public void include(EventRequest eventRequest, EventResponse eventResponse) throws PortletException, IOException {
    CustomRequestWrapper requestWrapper = null;    
    PortletContainerConf conf = (PortletContainerConf) cont.
        getComponentInstanceOfType(PortletContainerConf.class);
    try {      
      requestWrapper = 
        ((CustomRequestWrapper) ((HttpServletRequestWrapper)eventRequest).getRequest());
/*
      CustomResponseWrapper responseWrapper = 
        (CustomResponseWrapper) ((HttpServletResponseWrapper)response).getResponse();
*/
      CustomResponseWrapper realResponseWrapper = 
        (CustomResponseWrapper) ((HttpServletResponseWrapper)eventResponse).getResponse();
      NestedResponseWrapper responseWrapper = new NestedResponseWrapper(realResponseWrapper);
            
      eventRequest.setAttribute("javax.portlet.config",((EventRequestImp)eventRequest).getPortletConfig());
      eventRequest.setAttribute("javax.portlet.request", eventRequest);
      eventRequest.setAttribute("javax.portlet.response", eventResponse);
      eventRequest.setAttribute(PortletRequest.LIFECYCLE_PHASE, PortletRequest.EVENT_PHASE);
      responseWrapper.flushBuffer();
      requestWrapper.setRedirected(true);
      requestWrapper.setRedirectedPath(path);
      requestWrapper.setContextPath(eventRequest.getContextPath());
      if (dispatcher != null)
        dispatcher.include(requestWrapper, responseWrapper);
      char[] ca = responseWrapper.getPortletContent();
      if (ca == null)
        ca = new char[0];
      if (realResponseWrapper.isStreamUsed()) {
        realResponseWrapper.getOutputStream().write((new String(ca)).getBytes());
      } else
        realResponseWrapper.getWriter().write(ca);
    } catch (ServletException e) {
      if (e.getRootCause() != null)
        log.error("Root cause of the exception", e.getRootCause());
      log.error("Problems occur when using PortletDispatcher", e);
      throw new PortletException("Problems occur when using PortletDispatcher", e);
    } finally {
      eventRequest.removeAttribute("javax.portlet.config");
      eventRequest.removeAttribute("javax.portlet.request");
      eventRequest.removeAttribute("javax.portlet.response");      
      eventRequest.removeAttribute(PortletRequest.LIFECYCLE_PHASE); 
      if(requestWrapper != null)
        requestWrapper.setRedirected(false);
    }   
  }
  
//  public void include(FragmentRequest fragmentRequest, FragmentResponse fragmentResponse) throws PortletException, IOException {
//
//    CustomRequestWrapper requestWrapper = null;    
//    PortletContainerConf conf = (PortletContainerConf) cont.
//        getComponentInstanceOfType(PortletContainerConf.class);
//    try {      
//      requestWrapper = 
//        ((CustomRequestWrapper) ((HttpServletRequestWrapper)fragmentRequest).getRequest());
///*
//      CustomResponseWrapper responseWrapper = 
//        (CustomResponseWrapper) ((HttpServletResponseWrapper)response).getResponse();
//*/
//      CustomResponseWrapper realResponseWrapper = 
//        (CustomResponseWrapper) ((HttpServletResponseWrapper)fragmentResponse).getResponse();
//      NestedResponseWrapper responseWrapper = new NestedResponseWrapper(realResponseWrapper);
//            
//      fragmentRequest.setAttribute("javax.portlet.config",((FragmentRequestImp)fragmentRequest).getPortletConfig());
//      fragmentRequest.setAttribute("javax.portlet.request", fragmentRequest);
//      fragmentRequest.setAttribute("javax.portlet.response", fragmentResponse);
//      fragmentRequest.setAttribute(PortletRequest.LIFECYCLE_PHASE, PortletRequest.FRAGMENT_SERVING_PHASE);
//      responseWrapper.flushBuffer();
//      requestWrapper.setRedirected(true);
//      requestWrapper.setRedirectedPath(path);
//      requestWrapper.setContextPath(fragmentRequest.getContextPath());
//      if (dispatcher != null)
//        dispatcher.include(requestWrapper, responseWrapper);
//      char[] ca = responseWrapper.getPortletContent();
//      if (ca == null)
//        ca = new char[0];
//      if (realResponseWrapper.isStreamUsed()) {
//        realResponseWrapper.getOutputStream().write((new String(ca)).getBytes());
//      } else
//        realResponseWrapper.getWriter().write(ca);
//    } catch (ServletException e) {
//      if (e.getRootCause() != null)
//        log.error("Root cause of the exception", e.getRootCause());
//      log.error("Problems occur when using PortletDispatcher", e);
//      throw new PortletException("Problems occur when using PortletDispatcher", e);
//    } finally {
//      fragmentRequest.removeAttribute("javax.portlet.config");
//      fragmentRequest.removeAttribute("javax.portlet.request");
//      fragmentRequest.removeAttribute("javax.portlet.response");   
//      fragmentRequest.removeAttribute(PortletRequest.LIFECYCLE_PHASE); 
//      if(requestWrapper != null)
//        requestWrapper.setRedirected(false);
//    }   
//  }
  
//  public void forward(FragmentRequest fragmentRequest, FragmentResponse fragmentResponse) throws PortletException, IOException, IllegalStateException {
//    try {      
//      ServletContext portalContext = (ServletContext) cont.getComponentInstanceOfType(ServletContext.class);
//      ServletContext portletContext = portalContext.getContext(fragmentRequest.getContextPath());
//      RequestDispatcher dispatcher = portletContext.getRequestDispatcher(path);
//
//      ServletRequest servRequest = ((CustomRequestWrapper) ((PortletRequestImp) (fragmentRequest)).getRequest()).getRequest();
//      ServletResponse servResponse = ((CustomResponseWrapper) ((PortletResponseImp) (fragmentResponse)).getResponse()).getResponse();
//
//      if (dispatcher != null)
//        dispatcher.forward(servRequest, servResponse);
//
//    } catch (ServletException e) {
//      if (e.getRootCause() != null)
//        log.error("Root cause of the exception", e.getRootCause());
//      log.error("Problems occur when using PortletDispatcher", e);
//      throw new PortletException("Problems occur when using PortletDispatcher", e);
//    } finally {
//    }  
//  }
  
  public void forward(ResourceRequest resourceRequest, ResourceResponse resourceResponse) throws PortletException, IOException, IllegalStateException {
    try {      
      ServletContext portalContext = (ServletContext) cont.getComponentInstanceOfType(ServletContext.class);
      ServletContext portletContext = portalContext.getContext(resourceRequest.getContextPath());
      RequestDispatcher dispatcher = portletContext.getRequestDispatcher(path);

      ServletRequest servRequest = ((CustomRequestWrapper) ((PortletRequestImp) (resourceRequest)).getRequest()).getRequest();
      ServletResponse servResponse = ((CustomResponseWrapper) ((PortletResponseImp) (resourceResponse)).getResponse()).getResponse();

      if (dispatcher != null)
        dispatcher.forward(servRequest, servResponse);

    } catch (ServletException e) {
      if (e.getRootCause() != null)
        log.error("Root cause of the exception", e.getRootCause());
      log.error("Problems occur when using PortletDispatcher", e);
      throw new PortletException("Problems occur when using PortletDispatcher", e);
    } finally {
    }    
  }

}