/**
 * Copyright 2001-2003 The eXo Platform SARL All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

/**
 * Created by The eXo Platform SARL
 * Author : Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 * Date: Jul 27, 2003
 * Time: 8:44:32 PM
 */
package org.exoplatform.services.portletcontainer.plugins.pc.portletAPIImp;


import javax.portlet.PortletRequest ;
import javax.portlet.PortalContext  ;
import javax.portlet.PortletPreferences ;
import javax.portlet.PortletMode ;
import javax.portlet.WindowState ;
import javax.portlet.PortletSession ;
import javax.portlet.PortletConfig ;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpSession;
import java.io.UnsupportedEncodingException;
import java.util.*;

import org.apache.commons.logging.Log;
import org.exoplatform.Constants;
import org.exoplatform.container.ExoContainer;
import org.exoplatform.services.log.LogUtil;
import org.exoplatform.services.portletcontainer.ExoPortletRequest;
import org.exoplatform.services.portletcontainer.helper.PortletWindowInternal;
import org.exoplatform.services.portletcontainer.pci.*;
import org.exoplatform.services.portletcontainer.pci.model.*;
import org.exoplatform.services.portletcontainer.plugins.pc.PortletApplicationProxy;
import org.exoplatform.services.portletcontainer.plugins.pc.PortletContainerConf;
import org.exoplatform.services.portletcontainer.plugins.pc.PortletContainerDispatcher;

//import org.exoplatform.services.portletcontainer.plugins.pc.Constants;



/**
 * This implementation acts like a wrapper to the global ServletRequest
 *
 * This object should be pooled, therefore two fill and empty methods
 * are provided.
 *
 */
public abstract class PortletRequestImp extends HttpServletRequestWrapper
    implements PortletRequest, ExoPortletRequest, Map {

  private Log log_;
  protected ExoContainer cont;
  protected RequestContext reqCtx;
  private Vector paramNames;
  private Map filteredMap;
  protected boolean encodingModified;
  protected String enc;
  protected Map decodedParameters;

  public PortletRequestImp(RequestContext reqCtx) {
    super(reqCtx.getHttpServletRequest());
    this.log_ = LogUtil.getLog("org.exoplatform.services.portletcontainer");
    this.cont = ((PortletContextImpl)reqCtx.getPortletContext()).getContainer();
		this.reqCtx = reqCtx;
    this.enc = reqCtx.getHttpServletRequest().getCharacterEncoding();
    decodedParameters = new HashMap();
  }

  public boolean isWindowStateAllowed(WindowState windowState) {
    Enumeration e = reqCtx.getPortalContext().getSupportedWindowStates();
    while (e.hasMoreElements()) {
      WindowState supportedWindowState = (WindowState) e.nextElement();
      if (supportedWindowState.equals(windowState))
        return true;
    }
    return false;
  }

  public boolean isPortletModeAllowed(PortletMode portletMode) {
    Enumeration e = reqCtx.getPortalContext().getSupportedPortletModes();
    while (e.hasMoreElements()) {
      PortletMode supportedPortletMode = (PortletMode) e.nextElement();
      if (supportedPortletMode.toString().toLowerCase().equals(portletMode.toString().toLowerCase()))
        return true;
    }             
    return false;
  }

  public PortletConfig getPortletConfig(){
    ExoContainer manager = cont;
    String portletAppName = reqCtx.getPortletWindowInternal().getWindowID().getPortletApplicationName() ;
    PortletApplicationProxy proxy = 
      (PortletApplicationProxy) manager.getComponentInstance(portletAppName);
    return proxy.getPortletConfig(reqCtx.getPortletWindowInternal().getWindowID().getPortletName());
  }

  public Input getInput() {
    return reqCtx.getInput();
  }

  public PortletMode getPortletMode() {
    return reqCtx.getInput().getPortletMode();
  }

  public WindowState getWindowState() {
    return reqCtx.getInput().getWindowState();
  }

  public PortletPreferences getPreferences() {
    return reqCtx.getPortletWindowInternal().getPreferences();
  }

  public PortletSession getPortletSession() {
    return getPortletSession(true);
  }

  public PortletSession getPortletSession(boolean create) {
    PortletContainerConf conf = (PortletContainerConf)cont.
        getComponentInstanceOfType(PortletContainerConf.class);
    if(create){
      if(reqCtx.getSession().isSessionValid()){
        return reqCtx.getSession();
      }
    	reqCtx.getSession().setSession(((HttpServletRequest)super.getRequest()).getSession());          
      return reqCtx.getSession();             
    }
    HttpSession tmpSession = ((HttpServletRequest)super.getRequest()).getSession(false);                
    if(tmpSession == null)
    	return null;
    //needed for Tomcat 5.0.16 (first stable version)  
    try{
    	tmpSession.getLastAccessedTime();  
    } catch (IllegalStateException e){
    	log_.error("IllegalStateExcetion sent in PortletRequestImp getPortletSession()", e);
    	return null;
    }                            
    return reqCtx.getSession();            
  }

  public boolean isRequestedSessionIdValid() {
		PortletContainerConf conf = 
      (PortletContainerConf) cont.
                             getComponentInstanceOfType(PortletContainerConf.class);
		try {
			((HttpServletRequest) super.getRequest()).getSession()
					.getLastAccessedTime();
		} catch (IllegalStateException e) {
			log_.error("IllegalStateExcetion sent in PortletRequestImp isRequestedSessionIdValid()",	e);
			return false;
		}
		return ((HttpServletRequest) super.getRequest()).isRequestedSessionIdValid();
	}

  public String getProperty(String s) {
    String header = ((HttpServletRequest)super.getRequest()).getHeader(s);
    if(header != null)
      return header;
    return reqCtx.getPortalContext().getProperty(s);
  }

  public Enumeration getProperties(String s) {
    Enumeration header = ((HttpServletRequest)super.getRequest()).getHeaders(s);
    return header;
  }

  public Enumeration getPropertyNames() {
    Enumeration headerNames = ((HttpServletRequest)super.getRequest()).getHeaderNames();
    Enumeration portalPropertyNames = reqCtx.getPortalContext().getPropertyNames();
    Collection global = new ArrayList();
    while (portalPropertyNames.hasMoreElements()) {
      String s = (String) portalPropertyNames.nextElement();
      global.add(s);
    }
    while (headerNames.hasMoreElements()) {
      String s = (String) headerNames.nextElement();
      global.add(s);
    }
    return Collections.enumeration(global);
  }

  public PortalContext getPortalContext() {
    return reqCtx.getPortalContext();
  }

  public String getResponseContentType() {
    List l = reqCtx.getPortletDatas().getSupports();
    String markup = reqCtx.getInput().getMarkup() ;
    String inputPortletMode = reqCtx.getInput().getPortletMode().toString() ;
    for (int i = 0; i < l.size(); i++) {
      Supports supportsType = (Supports) l.get(i);
      String mimeType = supportsType.getMimeType();
      if (mimeType.equals(markup)) {
        List portletModes = supportsType.getPortletMode();
        for (int modeIdx  = 0; modeIdx < portletModes.size(); modeIdx++) {          
          String  portletMode = (String) portletModes.get(modeIdx);
          if(portletMode.equals(inputPortletMode))  return mimeType;
        }        
      }        
    }
    return "text/html";
  }

  public Enumeration getResponseContentTypes() {
    List c = new ArrayList();
    c.add(getResponseContentType());
    String markup = reqCtx.getInput().getMarkup() ;
    String inputPortletMode = reqCtx.getInput().getPortletMode().toString() ;
    for (Iterator iter = reqCtx.getSupportedContents().iterator(); iter.hasNext();) {
      String supportedContent = (String) iter.next();
      List l = reqCtx.getPortletDatas().getSupports();
      for (int i = 0; i < l.size(); i++) {
        Supports supportsType = (Supports) l.get(i);
        String mimeType = supportsType.getMimeType();
        if(supportedContent.equals(mimeType) && !supportedContent.equals(markup)){
          List portletModes = supportsType.getPortletMode();
          for (Iterator iter2 = portletModes.iterator(); iter2.hasNext();) {
            String portletMode = (String) iter2.next();
            if(portletMode.equals(inputPortletMode)){
              c.add(mimeType);
              break;
            }
          }          
        }        
      }      
    }
    return Collections.enumeration(c);
  }

  public boolean isUserInRole(String role){    
    List l = reqCtx.getPortletDatas().getSecurityRoleRef();
    for (Iterator iterator = l.iterator(); iterator.hasNext();) {
      SecurityRoleRef securityRoleRef = (SecurityRoleRef) iterator.next();
      if(securityRoleRef.getRoleName().equals(role)){
        String roleLink = securityRoleRef.getRoleLink();
        if(roleLink == null || "".equals(roleLink)){
          if(isRoleDefinedInWebXML(role)) return super.isUserInRole(role);
          return false;
        }
        if(isRoleDefinedInWebXML(roleLink))	return super.isUserInRole(roleLink);
        return false;
      }
    }
    return false;
  }

  private boolean isRoleDefinedInWebXML(String role){      
    for (Iterator iter = reqCtx.getRoles().iterator(); iter.hasNext();) {      
      String roleDefined = (String) iter.next();                 
      if(roleDefined.equals(role))
        return true;
    }
    return false;
  }

  public Portlet getPortletDatas() {
    return reqCtx.getPortletDatas();
  }

  public PortletWindowInternal getPortletWindowInternal() {
    return reqCtx.getPortletWindowInternal();
  }

  public boolean needsSecurityContraints(String portletName) {
    for (Iterator iterator = reqCtx.getSecurityContraints().iterator(); iterator.hasNext();) {
      SecurityConstraint securityConstraint = (SecurityConstraint) iterator.next();
      List l = securityConstraint.getPortletCollection().getPortletName();
      for (Iterator iterator2 = l.iterator(); iterator2.hasNext();) {
        String portletN = (String) iterator2.next();
        if(portletN.equals(portletName))  return true;
      }
    }
    return false;
  }

  public String getAuthType(){
    String type = super.getAuthType();
    if(HttpServletRequest.BASIC_AUTH.equals(type)) 
      return PortletRequest.BASIC_AUTH;
    else if (HttpServletRequest.DIGEST_AUTH.equals(type))
      return PortletRequest.DIGEST_AUTH;
    else if (HttpServletRequest.CLIENT_CERT_AUTH.equals(type))
      return PortletRequest.CLIENT_CERT_AUTH;    	
    else if (HttpServletRequest.FORM_AUTH.equals(type))
      return PortletRequest.FORM_AUTH;
    else
      return type;             	
  }

  public Object getAttribute(String name) {
    if (name == null) {
      throw new IllegalArgumentException("The attribute name cannot be null") ;
    }
    return super.getAttribute(name) ;
  }  

  public void setAttribute(String name, Object value) {
    if (name == null) {
      throw new IllegalArgumentException("The attribute name cannot be null") ;
    }
    //when the value is null, should have the same effect as removeAttribute (Spec)
    if(value == null) {
      super.removeAttribute(name) ;
    } else {
      super.setAttribute(name, value) ;
    }
  }

  public void removeAttribute(String name) {
    if (name == null) {
      throw new IllegalArgumentException("The attribute name cannot be null") ;
    }
    super.removeAttribute(name) ;
  }

  public String getContextPath() {
    return "/" +  this.reqCtx.getPortletContext().getPortletContextName() ;
  }

  // Bridge methods

  public int size() {
    int n = 0;
    Enumeration keys = getAttributeNames();
    while (keys.hasMoreElements()) {
      //String key = (String) keys.nextElement();
      n++;
    }
    return n;
  }

  public boolean isEmpty() {
    return !getAttributeNames().hasMoreElements();
  }

  public boolean containsKey( Object key ) {
    return (getAttribute( (String) key ) != null);
  }

  public boolean containsValue( Object value ) {
    boolean match = false;
    Enumeration keys = getAttributeNames();
    while (keys.hasMoreElements()) {
      String key = (String) keys.nextElement();
      Object val = getAttribute( key );
      if (value.equals( val )) {
        match = true;
        break;
      }
    }
    return match;
  }

  public Object get( Object key ) {
    return getAttribute( (String) key );
  }

  public Object put( Object key, Object value ) {
    Object result = null;
    if (containsKey( key )) result = getAttribute( (String) key );
    setAttribute( (String) key, value );
    return result;
  }

  public Object remove( Object key ) {
    Object result = null;
    if (containsKey( key )) result = getAttribute( (String) key );
    removeAttribute( (String) key );
    return result;
  }

  public void putAll( Map t ) {
    for (Iterator i = t.keySet().iterator(); i.hasNext();) {
      String key = (String) i.next();
      Object value = t.get( key );
      put( key, value );
    }
  }

  public void clear() {
    throw new UnsupportedOperationException();
  }

  public Set keySet() {
    throw new UnsupportedOperationException();
  }

  public Collection values() {
    throw new UnsupportedOperationException();
  }

  public Set entrySet() {
    throw new UnsupportedOperationException();
  }
  
  public Locale getLocale(){
    List locales = reqCtx.getInput().getLocales();
    if(locales != null && !locales.isEmpty()){
      return (Locale) locales.iterator().next();
    }          
    return super.getLocale();
  }
  
  public Enumeration getLocales(){
    List locales = reqCtx.getInput().getLocales();
    if(locales != null){
      return Collections.enumeration(locales);
    }          
    return super.getLocales();
  }

  public WindowID getWindowID() {
    return reqCtx.getInput().getWindowID();
  }

  //adds for jsr-286
  public String getWindowId() {
    return reqCtx.getInput().getWindowID().toString();
  }
  
  public String getParameter(String param) {
    if (param == null || param.startsWith(Constants.PARAMETER_ENCODER))
      return null;
    if (decodedParameters.containsKey(param)){      
      if (decodedParameters.get(param) instanceof String) {
        return (String) decodedParameters.get(param);
      } else {
        Object paramValue = decodedParameters.get(param);
        if(paramValue == null)
          return null;
        return ((String[]) paramValue)[0];
      }
    }
    
    Object obj = super.getParameter(param);    
    if (obj instanceof String[]) {
      String[] tmp = decode((String[]) obj);
      decodedParameters.put(param, tmp);
      return tmp[0];
    } else {
      String tmp = decode((String) obj);      
      decodedParameters.put(param, tmp);
      return tmp;
    }
  }

  public Enumeration getParameterNames() {
    if (paramNames == null) {
      Enumeration e = super.getParameterNames();
      paramNames = new Vector();
      while (e.hasMoreElements()) {
        String key = (String) e.nextElement();
        if (!key.startsWith(Constants.PARAMETER_ENCODER))
          paramNames.add(key);
      }
    }
    return paramNames.elements();
  }

  public String[] getParameterValues(String s) {
    if (s == null || s.startsWith(Constants.PARAMETER_ENCODER))
      return null;
    if (decodedParameters.containsKey(s)) {
      if (decodedParameters.get(s) instanceof String[]) {
        return (String[]) decodedParameters.get(s);
      } else {
        String[] array = { (String) decodedParameters.get(s) };
        return array;
      }
    }
    String[] tmp = decode(super.getParameterValues(s));
    decodedParameters.put(s, tmp);
    return tmp;
  }

  public Map getParameterMap() {
    if (filteredMap == null) {
      filteredMap = new HashMap();
      Map requestMap = super.getParameterMap();
      Set keySet = requestMap.keySet();
      Iterator names = keySet.iterator();
      while (names.hasNext()) {
        String name = (String) names.next();
        String[] obj = (String[]) requestMap.get(name);
        if (!name.startsWith(Constants.PARAMETER_ENCODER)) {
          filteredMap.put(name, decode(obj));
        }
      }
    }
    return Collections.unmodifiableMap(filteredMap);
  }

  private String[] decode(String[] tmp) {
    if(tmp == null)
      return null;
    if(!encodingModified)
      return tmp;
    for (int i = 0; i < tmp.length; i++) {
      try {
        tmp[i] = new String(tmp[i].getBytes(), enc);
      } catch (UnsupportedEncodingException e) {
        log_.error(" -- ActionRequestImp.decode[]: " + e);
      }
    }
    return tmp;
  }

  private String decode(String tmp) {
    if(tmp == null)
      return null;
    if(!encodingModified)
      return tmp;
    try {
      tmp = new String(tmp.getBytes(), enc);
    } catch (UnsupportedEncodingException e) {
      log_.error(" -- ActionRequestImp.decode[]: " + e);
    }
    return tmp;
  }
  
}