/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.
 * Please look at license.txt in info directory for more license detail.
 **/

/**
 * Created by The eXo Platform SARL        .
 * Author : Mestrallet Benjamin
 *          benjmestrallet@users.sourceforge.net
 * Date: Oct 9, 2003
 * Time: 12:06:04 PM
 */
package org.exoplatform.services.portletcontainer.imp.Portlet168;


import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.exoplatform.services.portletcontainer.imp.EmptyResponse;
import org.exoplatform.services.portletcontainer.pci.ExoWindowID;
import org.exoplatform.test.mocks.servlet.MockHttpSession;
import org.exoplatform.test.mocks.servlet.MockServletRequest;
import org.exoplatform.test.mocks.servlet.MockServletResponse;
import java.util.Locale;

public class TestCaching extends BaseTest {

  public TestCaching(String s) {
    super(s);
  }

  public void testProgrammaticExpirationPeriodChange() throws PortletException {
    ((ExoWindowID)renderInput.getWindowID()).setPortletName("HelloWorld");
    HttpServletRequest request = new MockServletRequest(new MockHttpSession(), Locale.US, true);
    HttpServletResponse response = new MockServletResponse(new EmptyResponse());
    try {
      portletContainer.render(request, response, renderInput);
      assertEquals(6, portletMonitor.getCacheExpirationPeriod("war_template",
                                                              "HelloWorld"));
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void testSimpleCache(){
    ((ExoWindowID)renderInput.getWindowID()).setPortletName("HelloWorld");
    HttpServletRequest request = new MockServletRequest(new MockHttpSession(), Locale.US, true);
    HttpServletResponse response = new MockServletResponse(new EmptyResponse());
    try {
      portletContainer.render(request, response, renderInput);
      portletContainer.render(request, response, renderInput);
      Thread.sleep(8000);
      portletContainer.render(request, response, renderInput);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void testNotExpirableCache(){
    ((ExoWindowID)renderInput.getWindowID()).setPortletName("HelloWorld2");
    HttpServletRequest request = new MockServletRequest(new MockHttpSession(), Locale.US);
    HttpServletResponse response = new MockServletResponse(new EmptyResponse());
    try {
      portletContainer.render(request, response, renderInput);
      //the console output should appear once
      portletContainer.render(request, response, renderInput);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void testPortletWithNoCache(){
    ((ExoWindowID)renderInput.getWindowID()).setPortletName("PortletWithNoCacheTag");
    HttpServletRequest request = new MockServletRequest(new MockHttpSession(), Locale.US);
    HttpServletResponse response = new MockServletResponse(new EmptyResponse());
    try {
      portletContainer.render(request, response, renderInput);
      portletContainer.render(request, response, renderInput);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void testModeAndStateChange(){
    ((ExoWindowID)renderInput.getWindowID()).setPortletName("HelloWorld2");
    HttpServletRequest request = new MockServletRequest(new MockHttpSession(), Locale.US);
    HttpServletResponse response = new MockServletResponse(new EmptyResponse());
    try {
      portletContainer.render(request, response, renderInput);
      portletContainer.render(request, response, renderInput);
      renderInput.setPortletMode(PortletMode.EDIT);
      portletContainer.render(request, response, renderInput);
      portletContainer.render(request, response, renderInput);
      renderInput.setWindowState(WindowState.MAXIMIZED);
      portletContainer.render(request, response, renderInput);
      portletContainer.render(request, response, renderInput);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void testRenderURLRequest(){
    ((ExoWindowID)renderInput.getWindowID()).setPortletName("HelloWorld2");
    renderInput.setPortletMode(PortletMode.EDIT);
    HttpServletRequest request = new MockServletRequest(new MockHttpSession(), Locale.US);
    HttpServletResponse response = new MockServletResponse(new EmptyResponse());
    try {      
      portletContainer.render(request, response, renderInput);
      renderInput.setUpdateCache(true);
      portletContainer.render(request, response, renderInput);
      renderInput.setUpdateCache(false);
      portletContainer.render(request, response, renderInput);
      renderInput.setUpdateCache(true);
      portletContainer.render(request, response, renderInput);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void testGlobalCache(){
    ((ExoWindowID)renderInput.getWindowID()).setPortletName("PortletToTestGlobalCache");
    MockServletRequest request = new MockServletRequest(new MockHttpSession(), Locale.US);
    HttpServletResponse response = new MockServletResponse(new EmptyResponse());
    try {
      portletContainer.render(request, response, renderInput);
      //the console output should appear once
      portletContainer.render(request, response, renderInput);
      request.setRemoteUser("new_remote");
      portletContainer.render(request, response, renderInput);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
