package javax.portlet;

import java.security.Principal;
import java.util.*;
import javax.servlet.http.Cookie;

public interface PortletRequest {

  public static final String USER_INFO = "javax.portlet.userinfo";
  public static final String BASIC_AUTH = "BASIC";
  public static final String FORM_AUTH = "FORM";
  public static final String CLIENT_CERT_AUTH = "CLIENT_CERT";
  public static final String DIGEST_AUTH = "DIGEST";
  // CC/PP - Composite Capabilities/Preference Profiles
  public static final String CCPP_PROFILE = "javax.portlet.ccpp";
  // P3P - Platform for Privacy Preferences 1.0 (user information constants)
  public enum P3PUserInfos {
    USER_BDATE_YMD_YEAR("user.bdate.ymd.year"), USER_BDATE_YMD_MONTH("user.bdate.ymd.month"),
    USER_BDATE_YMD_DAY("user.bdate.ymd.day"), USER_BDATE_HMS_HOUR("user.bdate.hms.hour"),
    USER_BDATE_HMS_MINUTE("user.bdate.hms.minute"), USER_BDATE_HMS_SECOND("user.bdate.hms.second"),
    USER_BDATE_FRACTIONSECOND("user.bdate.fractionsecond"), USER_BDATE_TIMEZONE("user.bdate.timezone"),
    USER_GENDER("user.gender"), USER_EMPLOYER("user.employer"),
    USER_DEPARTMENT("user.department"), USER_JOBTITLE("user.jobtitle"),
    USER_NAME_PREFIX("user.name.prefix"), USER_NAME_GIVEN("user.name.given"),
    USER_NAME_FAMILY("user.name.family"), USER_NAME_MIDDLE("user.name.middle"),
    USER_NAME_SUFFIX("user.name.suffix"), USER_NAME_NICKNAME("user.name.nickName"),
    USER_LOGIN_ID("user.login.id"),
    USER_HOMEINFO_POSTAL_NAME("user.home-info.postal.name"),
    USER_HOMEINFO_POSTAL_STREET("user.home-info.postal.street"),
    USER_HOMEINFO_POSTAL_CITY("user.home-info.postal.city"),
    USER_HOMEINFO_POSTAL_STATEPROV("user.home-info.postal.stateprov"),
    USER_HOMEINFO_POSTAL_POSTALCODE("user.home-info.postal.postalcode"),
    USER_HOMEINFO_POSTAL_COUNTRY("user.home-info.postal.country"),
    USER_HOMEINFO_POSTAL_ORGANIZATION("user.home-info.postal.organization"),
    USER_HOMEINFO_TELECOM_TELEPHONE_INTCODE("user.home-info.telecom.telephone.intcode"),
    USER_HOMEINFO_TELECOM_TELEPHONE_LOCCODE("user.home-info.telecom.telephone.loccode"),
    USER_HOMEINFO_TELECOM_TELEPHONE_NUMBER("user.home-info.telecom.telephone.number"),
    USER_HOMEINFO_TELECOM_TELEPHONE_EXT("user.home-info.telecom.telephone.ext"),
    USER_HOMEINFO_TELECOM_TELEPHONE_COMMENT("user.home-info.telecom.telephone.comment"),
    USER_HOMEINFO_TELECOM_FAX_INTCODE("user.home-info.telecom.fax.intcode"),
    USER_HOMEINFO_TELECOM_FAX_LOCCODE("user.home-info.telecom.fax.loccode"),
    USER_HOMEINFO_TELECOM_FAX_NUMBER("user.home-info.telecom.fax.number"),
    USER_HOMEINFO_TELECOM_FAX_EXT("user.home-info.telecom.fax.ext"),
    USER_HOMEINFO_TELECOM_FAX_COMMENT("user.home-info.telecom.fax.comment"),
    USER_HOMEINFO_TELECOM_MOBILE_INTCODE("user.home-info.telecom.mobile.intcode"),
    USER_HOMEINFO_TELECOM_MOBILE_LOCCODE("user.home-info.telecom.mobile.loccode"),
    USER_HOMEINFO_TELECOM_MOBILE_NUMBER("user.home-info.telecom.mobile.number"),
    USER_HOMEINFO_TELECOM_MOBILE_EXT("user.home-info.telecom.mobile.ext"),
    USER_HOMEINFO_TELECOM_MOBILE_COMMENT("user.home-info.telecom.mobile.comment"),
    USER_HOMEINFO_TELECOM_PAGER_INTCODE("user.home-info.telecom.pager.intcode"),
    USER_HOMEINFO_TELECOM_PAGER_LOCCODE("user.home-info.telecom.pager.loccode"),
    USER_HOMEINFO_TELECOM_PAGER_NUMBER("user.home-info.telecom.pager.number"),
    USER_HOMEINFO_TELECOM_PAGER_EXT("user.home-info.telecom.pager.ext"),
    USER_HOMEINFO_TELECOM_PAGER_COMMENT("user.home-info.telecom.pager.comment"),
    USER_HOMEINFO_ONLINE_EMAIL("user.home-info.online.email"),
    USER_HOMEINFO_ONLINE_URI("user.home-info.online.uri"),
    USER_BUSINESSINFO_POSTAL_NAME("user.business-info.postal.name"),
    USER_BUSINESSINFO_POSTAL_STREET("user.business-info.postal.street"),
    USER_BUSINESSINFO_POSTAL_CITY("user.business-info.postal.city"),
    USER_BUSINESSINFO_POSTAL_STATEPROV("user.business-info.postal.stateprov"),
    USER_BUSINESSINFO_POSTAL_POSTALCODE("user.business-info.postal.postalcode"),
    USER_BUSINESSINFO_POSTAL_COUNTRY("user.business-info.postal.country"),
    USER_BUSINESSINFO_POSTAL_ORGANIZATION("user.business-info.postal.organization"),
    USER_BUSINESSINFO_TELECOM_TELEPHONE_INTCODE("user.business-info.telecom.telephone.intcode"),
    USER_BUSINESSINFO_TELECOM_TELEPHONE_LOCCODE("user.business-info.telecom.telephone.loccode"),
    USER_BUSINESSINFO_TELECOM_TELEPHONE_NUMBER("user.business-info.telecom.telephone.number"),
    USER_BUSINESSINFO_TELECOM_TELEPHONE_EXT("user.business-info.telecom.telephone.ext"),
    USER_BUSINESSINFO_TELECOM_TELEPHONE_COMMENT("user.business-info.telecom.telephone.comment"),
    USER_BUSINESSINFO_TELECOM_FAX_INTCODE("user.business-info.telecom.fax.intcode"),
    USER_BUSINESSINFO_TELECOM_FAX_LOCCODE("user.business-info.telecom.fax.loccode"),
    USER_BUSINESSINFO_TELECOM_FAX_NUMBER("user.business-info.telecom.fax.number"),
    USER_BUSINESSINFO_TELECOM_FAX_EXT("user.business-info.telecom.fax.ext"),
    USER_BUSINESSINFO_TELECOM_FAX_COMMENT("user.business-info.telecom.fax.comment"),
    USER_BUSINESSINFO_TELECOM_MOBILE_INTCODE("user.business-info.telecom.mobile.intcode"),
    USER_BUSINESSINFO_TELECOM_MOBILE_LOCCODE("user.business-info.telecom.mobile.loccode"),
    USER_BUSINESSINFO_TELECOM_MOBILE_NUMBER("user.business-info.telecom.mobile.number"),
    USER_BUSINESSINFO_TELECOM_MOBILE_EXT("user.business-info.telecom.mobile.ext"),
    USER_BUSINESSINFO_TELECOM_MOBILE_COMMENT("user.business-info.telecom.mobile.comment"),
    USER_BUSINESSINFO_TELECOM_PAGER_INTCODE("user.business-info.telecom.pager.intcode"),
    USER_BUSINESSINFO_TELECOM_PAGER_LOCCODE("user.business-info.telecom.pager.loccode"),
    USER_BUSINESSINFO_TELECOM_PAGER_NUMBER("user.business-info.telecom.pager.number"),
    USER_BUSINESSINFO_TELECOM_PAGER_EXT("user.business-info.telecom.pager.ext"),
    USER_BUSINESSINFO_TELECOM_PAGER_COMMENT("user.business-info.telecom.pager.comment"),
    USER_BUSINESSINFO_ONLINE_EMAIL("user.business-info.online.email"),
    USER_BUSINESSINFO_ONLINE_URI("user.business-info.online.uri");
    P3PUserInfos(String value) {this.value = value; }
    private final String value;
    public String toString() {return value;}
  }

  public static final String ACTION_PHASE = "ACTION_PHASE";
  public static final String EVENT_PHASE = "EVENT_PHASE";
  public static final String RENDER_PHASE = "RENDER_PHASE";
  public static final String RESOURCE_SERVING_PHASE = "RESOURCE_SERVING_PHASE";
  public static final String FRAGMENT_SERVING_PHASE = "FRAGMENT_SERVING_PHASE";

  public static final String RENDER_PART = "javax.portlet.render_part";
  public static final String RENDER_HEADERS = "RENDER_HEADERS";
  public static final String RENDER_MARKUP = "RENDER_MARKUP";
  
  public static final String LIFECYCLE_PHASE = "javax.portlet.lifecycle_phase";

  public abstract boolean isWindowStateAllowed(WindowState windowstate);

  public abstract boolean isPortletModeAllowed(PortletMode portletmode);

  public abstract PortletMode getPortletMode();

  public abstract WindowState getWindowState();

  public abstract PortletPreferences getPreferences();

  public abstract PortletSession getPortletSession();

  public abstract PortletSession getPortletSession(boolean create);

  public abstract String getProperty(String s);

  public abstract Enumeration getProperties(String s);

  public abstract Enumeration getPropertyNames();

  public abstract PortalContext getPortalContext();

  public abstract String getAuthType();

  public abstract String getContextPath();

  public abstract String getRemoteUser();

  public abstract Principal getUserPrincipal();

  public abstract boolean isUserInRole(String s);

  public abstract Object getAttribute(String s);

  public abstract Enumeration getAttributeNames();

  public abstract String getParameter(String s);

  public abstract Enumeration getParameterNames();

  public abstract String[] getParameterValues(String s);

  public abstract Map getParameterMap();

  public abstract boolean isSecure();

  public abstract void setAttribute(String s, Object obj);

  public abstract void removeAttribute(String s);

  public abstract String getRequestedSessionId();

  public abstract boolean isRequestedSessionIdValid();

  public abstract String getResponseContentType();

  public abstract Enumeration getResponseContentTypes();

  public abstract Locale getLocale();

  public abstract Enumeration getLocales();

  public abstract String getScheme();

  public abstract String getServerName();

  public abstract int getServerPort();

  public abstract String getWindowId();
  
  public abstract Cookie[] getCookies();
  
}
